def test_first_chunk():
    import crapssim as craps
    from crapssim.strategy import BetPassLine, PassLineOddsMultiplier

    table = craps.Table()
    your_strat = BetPassLine(5) + PassLineOddsMultiplier(2)

    table.add_player(strategy=your_strat)
    table.run(max_rolls=20, verbose=True)


def test_second_chunk():
    import crapssim as craps

    n_sim = 20
    bankroll = 300
    strategies = {
        "place68": craps.strategy.examples.PassLinePlace68(5),
        "ironcross": craps.strategy.examples.IronCross(5),
    }

    for i in range(n_sim):
        table = craps.Table()
        for s in strategies:
            table.add_player(bankroll, strategy=strategies[s], name=s)

        table.run(max_rolls=float("inf"), max_shooter=10, verbose=False)

        for p in table.players:
            print(f"{i}, {p.name}, {p.bankroll}, {bankroll}, {table.dice.n_rolls}")



import pytest

from crapssim.bet import PassLine
from crapssim.strategy import BetPassLine
from crapssim.table import Table, TableUpdate


@pytest.mark.parametrize(
    "point, last_roll, strat_info, bets_before, dice_result, bets_after",
    [
        (None, None, None, [], None, [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (6, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (6, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (None, 6, None, [], (5, 1), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (5, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (5, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (5, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (5, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (5, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (None, 5, None, [], (2, 3), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (5, 3, None, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (5, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (5, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (None, 5, None, [], (3, 2), [PassLine(amount=5.0)]),
        (4, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (4, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (4, 3, None, [PassLine(amount=5.0)], (2, 1), [PassLine(amount=5.0)]),
        (4, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (10, 11, None, [PassLine(amount=5.0)], (6, 5), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (None, 10, None, [], (5, 5), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (None, 8, None, [], (4, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (None, 3, None, [], (1, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (10, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (10, 11, None, [PassLine(amount=5.0)], (6, 5), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 11, None, [], (5, 6), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (6, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (6, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (6, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (6, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (6, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (6, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (6, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (6, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (6, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (6, 4, None, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (6, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (None, 6, None, [], (3, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (9, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (9, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (9, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (9, 3, None, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (9, 8, None, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (None, 12, None, [], (6, 6), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (None, 8, None, [], (5, 3), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (6, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (6, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (None, 11, None, [], (5, 6), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (8, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (8, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (8, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (8, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (8, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (6, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (None, 11, None, [], (5, 6), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (9, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (9, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (9, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (9, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (5, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (1, 6), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (5, 4, None, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (5, 3, None, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (5, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (5, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (5, 3, None, [PassLine(amount=5.0)], (2, 1), [PassLine(amount=5.0)]),
        (5, 8, None, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (5, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (5, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (None, 5, None, [], (3, 2), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (10, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (10, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (10, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (10, 3, None, [PassLine(amount=5.0)], (2, 1), [PassLine(amount=5.0)]),
        (None, 10, None, [], (6, 4), [PassLine(amount=5.0)]),
        (None, 12, None, [], (6, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (6, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (None, 9, None, [], (6, 3), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (9, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (9, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (9, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (None, 9, None, [], (6, 3), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (9, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (9, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (None, 9, None, [], (4, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (6, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (8, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (8, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (None, 3, None, [], (2, 1), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (8, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (8, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (8, 11, None, [PassLine(amount=5.0)], (6, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (None, 8, None, [], (2, 6), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (3, 4), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (8, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (None, 8, None, [], (4, 4), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (1, 6), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (10, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (None, 11, None, [], (6, 5), [PassLine(amount=5.0)]),
        (None, 3, None, [], (2, 1), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (10, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (1, 6), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (10, 11, None, [PassLine(amount=5.0)], (6, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (1, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (4, 4, None, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (4, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (4, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (4, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (4, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (4, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (4, 5, None, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (None, 4, None, [], (3, 1), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (None, 10, None, [], (6, 4), [PassLine(amount=5.0)]),
        (4, 4, None, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (8, 2, None, [PassLine(amount=5.0)], (1, 1), [PassLine(amount=5.0)]),
        (8, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (8, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (8, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (9, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (None, 12, None, [], (6, 6), [PassLine(amount=5.0)]),
        (8, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (None, 7, None, [], (2, 5), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (9, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (6, 1), [PassLine(amount=5.0)]),
        (None, 3, None, [], (2, 1), [PassLine(amount=5.0)]),
        (5, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (5, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (5, 4, None, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (None, 9, None, [], (3, 6), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (9, 3, None, [PassLine(amount=5.0)], (2, 1), [PassLine(amount=5.0)]),
        (9, 6, None, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (None, 9, None, [], (3, 6), [PassLine(amount=5.0)]),
        (None, 12, None, [], (6, 6), [PassLine(amount=5.0)]),
        (None, 11, None, [], (5, 6), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (6, 4), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (10, 5, None, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (10, 5, None, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (10, 4, None, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (None, 10, None, [], (5, 5), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (6, 10, None, [PassLine(amount=5.0)], (4, 6), [PassLine(amount=5.0)]),
        (6, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (None, 6, None, [], (5, 1), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (10, 10, None, [PassLine(amount=5.0)], (5, 5), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (10, 8, None, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (10, 11, None, [PassLine(amount=5.0)], (5, 6), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (10, 6, None, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (10, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (None, 7, None, [], (4, 3), [PassLine(amount=5.0)]),
        (9, 9, None, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (9, 5, None, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (9, 5, None, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (None, 9, None, [], (4, 5), [PassLine(amount=5.0)]),
        (None, 7, None, [], (5, 2), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (None, 6, None, [], (2, 4), [PassLine(amount=5.0)]),
        (6, 6, None, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (6, 12, None, [PassLine(amount=5.0)], (6, 6), [PassLine(amount=5.0)]),
        (6, 8, None, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (None, 6, None, [], (5, 1), [PassLine(amount=5.0)]),
    ],
)
def test_passline_integration(
    point, last_roll, strat_info, bets_before, dice_result, bets_after
):
    table = Table()
    table.add_player(
        bankroll=float("inf"), strategy=BetPassLine(5)
    )  # ADD STRATEGY HERE
    table.point.number = point
    table.last_roll = last_roll
    table.players[0].bets = bets_before
    table.dice.result = dice_result
    TableUpdate().run_strategies(table)
    assert table.players[0].bets == bets_after



import copy

import pytest

from crapssim.bet import Field, PassLine, Place
from crapssim.strategy.examples import Risk12
from crapssim.table import Table, TableUpdate


@pytest.mark.parametrize(
    "point, last_roll, strat_info, bets_before, dice_result, bets_after",
    [
        (None, None, None, [], None, [PassLine(amount=5.0), Field(amount=5.0)]),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (5, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (5, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (3, 3),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (2, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (6, 9, {"winnings": 0}, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (
            None,
            6,
            {"winnings": 0},
            [],
            (5, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            2,
            {"winnings": 0},
            [],
            (1, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            6,
            6,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (2, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            5,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (2, 3),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            9,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (3, 6),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (3, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (6, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (5, 2),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (3, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (4, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            3,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            10,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (4, 6),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (4, 5),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            9,
            10,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (5, 5),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (1, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (4, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            12,
            {"winnings": 0},
            [],
            (6, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            6,
            6,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (1, 5),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            10,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (4, 6),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            11,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (5, 6),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            9,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (3, 6),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            11,
            {"winnings": 0},
            [],
            (5, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 10},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            11,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (4, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            6,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (3, 3),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            10,
            {"winnings": 10},
            [Place(8, amount=6.0), Place(6, amount=6.0)],
            (6, 4),
            [
                Place(8, amount=6.0),
                Place(6, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [Place(8, amount=6.0), Place(6, amount=6.0), PassLine(amount=5.0)],
            (5, 5),
            [Place(8, amount=6.0), Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            2,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (5, 4),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            None,
            3,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 1),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            None,
            2,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 1),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            4,
            4,
            {"winnings": 5},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (2, 2),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            4,
            10,
            {"winnings": 5},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (6, 4),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            4,
            5,
            {"winnings": 5},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (3, 2),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (6, 8, {"winnings": 0}, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (6, 5, {"winnings": 0}, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (6, 9, {"winnings": 0}, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (6, 5, {"winnings": 0}, [PassLine(amount=5.0)], (4, 1), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (2, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (5, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (5, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            2,
            {"winnings": 0},
            [],
            (1, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            9,
            9,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (4, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            8,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (6, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            4,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            10,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            12,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            9,
            {"winnings": 5},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 6),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (3, 6),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            9,
            5,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (4, 1),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (4, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (2, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (2, 2),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (3, 2),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (6, 5),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (6, 4, {"winnings": 0}, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (6, 9, {"winnings": 0}, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (6, 9, {"winnings": 0}, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (
            6,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0)],
        ),
        (6, 8, {"winnings": 0}, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (6, 4, {"winnings": 0}, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (6, 4, {"winnings": 0}, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (
            None,
            5,
            {"winnings": 0},
            [],
            (1, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 5),
            [PassLine(amount=5.0)],
        ),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (3, 1), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            2,
            {"winnings": 0},
            [],
            (1, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            6,
            6,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (5, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            10,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (6, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            5,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (1, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (
            6,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0)],
        ),
        (
            None,
            6,
            {"winnings": 0},
            [],
            (5, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (8, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (8, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (3, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (5, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (6, 9, {"winnings": 0}, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (6, 3, {"winnings": 0}, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (
            None,
            6,
            {"winnings": 0},
            [],
            (4, 2),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (1, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            4,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 2),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (6, 3),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            9,
            3,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (2, 1),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            9,
            5,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (4, 1),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            9,
            5,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
            (1, 4),
            [Place(6, amount=6.0), Place(8, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (5, 3, {"winnings": 0}, [PassLine(amount=5.0)], (2, 1), [PassLine(amount=5.0)]),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 5),
            [PassLine(amount=5.0)],
        ),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 6),
            [PassLine(amount=5.0)],
        ),
        (
            None,
            5,
            {"winnings": 0},
            [],
            (4, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (5, 4),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 4),
            [
                Place(6, amount=6.0),
                Place(8, amount=6.0),
                PassLine(amount=5.0),
                Field(amount=5.0),
            ],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (8, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (
            8,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 5),
            [PassLine(amount=5.0)],
        ),
        (
            None,
            8,
            {"winnings": 0},
            [],
            (3, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (5, 2),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (8, 8, {"winnings": 0}, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (
            8,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 6),
            [PassLine(amount=5.0)],
        ),
        (8, 3, {"winnings": 0}, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (
            None,
            8,
            {"winnings": 0},
            [],
            (4, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (5, 9, {"winnings": 0}, [PassLine(amount=5.0)], (4, 5), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (2, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (6, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (5, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (3, 3),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (2, 2),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            8,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (6, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (5, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (3, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (4, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            3,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (2, 1),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (1, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            10,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (2, 2),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (3, 2), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (
            5,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (
            None,
            6,
            {"winnings": 0},
            [],
            (4, 2),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (8, 8, {"winnings": 0}, [PassLine(amount=5.0)], (2, 6), [PassLine(amount=5.0)]),
        (
            None,
            8,
            {"winnings": 0},
            [],
            (3, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (5, 1), [PassLine(amount=5.0)]),
        (6, 8, {"winnings": 0}, [PassLine(amount=5.0)], (4, 4), [PassLine(amount=5.0)]),
        (6, 3, {"winnings": 0}, [PassLine(amount=5.0)], (1, 2), [PassLine(amount=5.0)]),
        (6, 4, {"winnings": 0}, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (6, 5, {"winnings": 0}, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (
            None,
            6,
            {"winnings": 0},
            [],
            (5, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (2, 4), [PassLine(amount=5.0)]),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (1, 3), [PassLine(amount=5.0)]),
        (5, 9, {"winnings": 0}, [PassLine(amount=5.0)], (3, 6), [PassLine(amount=5.0)]),
        (
            5,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (5, 9, {"winnings": 0}, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (
            5,
            12,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 6),
            [PassLine(amount=5.0)],
        ),
        (
            5,
            11,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (6, 5),
            [PassLine(amount=5.0)],
        ),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (5, 3), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (
            5,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0)],
        ),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (4, 2), [PassLine(amount=5.0)]),
        (
            None,
            5,
            {"winnings": 0},
            [],
            (2, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            11,
            {"winnings": 0},
            [],
            (5, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            5,
            5,
            {"winnings": 10},
            [PassLine(amount=5.0)],
            (1, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            5,
            9,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (5, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            5,
            8,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (5, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            5,
            6,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (2, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 10},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (5, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            2,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            10,
            3,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            11,
            {"winnings": 0},
            [],
            (5, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            6,
            6,
            {"winnings": 10},
            [PassLine(amount=5.0)],
            (1, 5),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            6,
            10,
            {"winnings": 10},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (6, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 10},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            12,
            {"winnings": 0},
            [],
            (6, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            8,
            8,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (3, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0)],
        ),
        (
            8,
            4,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0)],
            (3, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0)],
        ),
        (
            None,
            8,
            {"winnings": 5},
            [Place(6, amount=6.0)],
            (3, 5),
            [Place(6, amount=6.0), PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (5, 5),
            [Place(6, amount=6.0), PassLine(amount=5.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (3, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            5,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (3, 2),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            10,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            11,
            {"winnings": 0},
            [],
            (5, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 10},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            8,
            8,
            {"winnings": 10},
            [PassLine(amount=5.0)],
            (3, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0)],
        ),
        (
            None,
            8,
            {"winnings": 10},
            [Place(6, amount=6.0)],
            (3, 5),
            [Place(6, amount=6.0), PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            5,
            5,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (1, 4),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            10,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (6, 4),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            2,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (1, 1),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            3,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (2, 1),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            11,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (6, 5),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            10,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (5, 5),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            5,
            9,
            {"winnings": 0},
            [Place(6, amount=6.0), PassLine(amount=5.0)],
            (5, 4),
            [Place(6, amount=6.0), PassLine(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (3, 4),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (1, 4), [PassLine(amount=5.0)]),
        (5, 4, {"winnings": 0}, [PassLine(amount=5.0)], (2, 2), [PassLine(amount=5.0)]),
        (5, 9, {"winnings": 0}, [PassLine(amount=5.0)], (5, 4), [PassLine(amount=5.0)]),
        (5, 6, {"winnings": 0}, [PassLine(amount=5.0)], (1, 5), [PassLine(amount=5.0)]),
        (
            None,
            5,
            {"winnings": 0},
            [],
            (2, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (6, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (5, 5, {"winnings": 0}, [PassLine(amount=5.0)], (2, 3), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (
            5,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (4, 6),
            [PassLine(amount=5.0)],
        ),
        (5, 9, {"winnings": 0}, [PassLine(amount=5.0)], (6, 3), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (6, 2), [PassLine(amount=5.0)]),
        (5, 8, {"winnings": 0}, [PassLine(amount=5.0)], (3, 5), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (1, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            2,
            {"winnings": 0},
            [],
            (1, 1),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            4,
            4,
            {"winnings": 5},
            [PassLine(amount=5.0)],
            (1, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            11,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (6, 5),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            5,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (2, 3),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            2,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 1),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            4,
            5,
            {"winnings": 5},
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
            (1, 4),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 5},
            [],
            (2, 5),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            9,
            9,
            {"winnings": 0},
            [PassLine(amount=5.0)],
            (3, 6),
            [PassLine(amount=5.0), Place(6, amount=6.0), Place(8, amount=6.0)],
        ),
        (
            9,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (2, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            9,
            10,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (6, 4),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            9,
            6,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0)],
            (1, 5),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            9,
            3,
            {"winnings": 0},
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
            (1, 2),
            [PassLine(amount=5.0), Place(8, amount=6.0), Place(6, amount=6.0)],
        ),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (6, 6, {"winnings": 0}, [PassLine(amount=5.0)], (3, 3), [PassLine(amount=5.0)]),
        (
            None,
            7,
            {"winnings": 0},
            [],
            (4, 3),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
        (
            None,
            11,
            {"winnings": 0},
            [],
            (5, 6),
            [PassLine(amount=5.0), Field(amount=5.0)],
        ),
    ],
)
def test_risk12_integration(
    point, last_roll, strat_info, bets_before, dice_result, bets_after
):
    table = Table()
    strategy = Risk12()
    table.point.number = point
    table.last_roll = last_roll
    if table.point.number in (4, 9, 10):
        strategy.pre_point_winnings = 10
    elif strat_info is not None and "winnings" in strat_info:
        strategy.pre_point_winnings = strat_info["winnings"]
    table.add_player(bankroll=float("inf"), strategy=strategy)  # ADD STRATEGY HERE
    table.players[0].bets = copy.copy(bets_before)
    table.dice.result = dice_result
    TableUpdate().run_strategies(table)
    assert table.players[0].bets == bets_after



"""Strategies that place a given bet if that bet of that type isn't currently placed and the bet
is allowed."""

import enum
import typing

from crapssim.bet import (
    All,
    Any7,
    Bet,
    Boxcars,
    Come,
    DontCome,
    DontPass,
    Field,
    Fire,
    HardWay,
    Hop,
    PassLine,
    Place,
    Small,
    Tall,
    Three,
    Two,
    Yo,
)
from crapssim.strategy.tools import (
    AddIfNewShooter,
    AddIfNotBet,
    AddIfPointOff,
    AddIfPointOn,
    AddIfTrue,
    Player,
    RemoveIfPointOff,
    RemoveIfTrue,
    Strategy,
)


class StrategyMode(enum.Enum):
    ADD_IF_NOT_BET = enum.auto()
    ADD_IF_POINT_OFF = enum.auto()
    ADD_IF_POINT_ON = enum.auto()
    ADD_IF_NEW_SHOOTER = enum.auto()
    ADD_OR_INCREASE = enum.auto()
    BET_IF_POINT_ON = enum.auto()
    REPLACE = enum.auto()


class _BaseSingleBet(Strategy):
    def __init__(
        self,
        bet: Bet,
        mode: StrategyMode = StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__()
        self.bet = bet
        self.mode = mode

    def completed(self, player: Player) -> bool:
        return player.bankroll < self.bet.amount and len(player.bets) == 0

    def update_bets(self, player: Player) -> None:
        if not self.bet.is_allowed(player):
            return

        match self.mode:
            case StrategyMode.ADD_IF_NOT_BET:
                AddIfNotBet(self.bet).update_bets(player)
            case StrategyMode.ADD_IF_POINT_ON:
                AddIfPointOn(self.bet).update_bets(player)
            case StrategyMode.ADD_IF_POINT_OFF:
                AddIfPointOff(self.bet).update_bets(player)
            case StrategyMode.ADD_IF_NEW_SHOOTER:
                AddIfNewShooter(self.bet).update_bets(player)
            case StrategyMode.ADD_OR_INCREASE:
                player.add_bet(self.bet)
            case StrategyMode.BET_IF_POINT_ON:
                AddIfPointOn(self.bet).update_bets(player)
                # If only betting when point on, also need to turn off when point off
                RemoveIfPointOff(self.bet).update_bets(player)
            case StrategyMode.REPLACE:
                existing_bets = player.already_placed_bets(self.bet)
                for bet in existing_bets:
                    player.remove_bet(bet)
                player.add_bet(self.bet)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(bet_amount={self.bet.amount},"
            f" mode={self.mode})"
        )


class BetPlace(Strategy):
    """Strategy that makes multiple Place bets of given amounts. It can also skip making the bet
    if the point is the same as the given bet number."""

    def __init__(
        self,
        place_bet_amounts: dict[int, float],
        mode: StrategyMode = StrategyMode.BET_IF_POINT_ON,
        skip_point: bool = True,
        skip_come: bool = False,
    ):
        """Strategy for making multiple place bets.

        Parameters
        ----------
        place_bet_amounts
            Dictionary of the point to make the Place bet on and the amount of the
            place bet to make.
        skip_point
            If True don't make the bet on the given Place if that's the number the tables Point
            is on.
        skip_come
            If True don't make the bet on the given Place if there is a Come bet with that Point
            already on that number.
        """
        super().__init__()
        self.place_bet_amounts = place_bet_amounts
        self.mode = mode
        self.skip_point = skip_point
        self.skip_come = skip_come

    def completed(self, player: Player) -> bool:
        """The strategy is completed if the player can no longer make any of the place bets in the
        place_bet_amounts dictionary and there are no Place bets on the table.

        Parameters
        ----------
        player
            The player to check the bankroll for

        Returns
        -------
        True if there are no Place bets on the table and the player can't make any more Place bets
        because their bankroll is too low.
        """
        return (
            player.bankroll < min(x for x in self.place_bet_amounts.values())
            and len([x for x in player.bets if isinstance(x, Place)]) == 0
        )

    def update_bets(self, player: Player) -> None:
        """Add the place bets on the numbers and amounts defined by place_bet_amounts.

        Parameters
        ----------
        player
            The player to add the place bets to.
        """
        if self.skip_point:
            self.remove_point_bet(player)

        for number, amount in self.place_bet_amounts.items():
            if self.skip_point and number == player.table.point.number:
                continue
            if self.skip_come:
                come_numbers = [
                    x.point.number for x in player.bets if isinstance(x, Come)
                ]
                if number in come_numbers:
                    continue
            _BaseSingleBet(Place(number, amount), mode=self.mode).update_bets(player)

    @staticmethod
    def remove_point_bet(player: Player) -> None:
        """If skip_point is true and the player has a place bet for the table point number,
        remove the Place bet.

        Parameters
        ----------
        player
            The player to check and see if they have the given bet.
        """
        point = player.table.point.number
        RemoveIfTrue(
            lambda b, p: isinstance(b, Place) and b.number == point
        ).update_bets(player)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(place_bet_amounts={self.place_bet_amounts},"
            f" mode={self.mode},"
            f" skip_point={self.skip_point}, skip_come={self.skip_come})"
        )


class BetPassLine(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_POINT_OFF,
    ):
        super().__init__(PassLine(bet_amount), mode=mode)


class BetDontPass(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_POINT_OFF,
    ):
        super().__init__(DontPass(bet_amount), mode=mode)


class BetCome(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_POINT_ON,
    ):
        super().__init__(Come(bet_amount), mode=mode)


class BetDontCome(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_POINT_ON,
    ):
        super().__init__(DontCome(bet_amount), mode=mode)


class BetHardWay(_BaseSingleBet):
    def __init__(
        self,
        number: tuple[int],
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        if number not in [4, 6, 8, 10]:
            raise NotImplementedError
        self.number = number
        super().__init__(HardWay(number, bet_amount), mode=mode)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}({self.number}, bet_amount={self.bet.amount},"
            f" mode={self.mode})"
        )


class BetHop(_BaseSingleBet):
    def __init__(
        self,
        result: tuple[int, int],
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        if result[0] not in [1, 2, 3, 4, 5, 6] or result[1] not in [1, 2, 3, 4, 5, 6]:
            raise NotImplementedError
        self.result: tuple[int, int] = tuple(sorted(result))
        super().__init__(Hop(result, bet_amount), mode=mode)

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}({self.result}, bet_amount={self.bet.amount},"
            f" mode={self.mode})"
        )


class BetField(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Field(bet_amount), mode=mode)


class BetAny7(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Any7(bet_amount), mode=mode)


class BetTwo(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Two(bet_amount), mode=mode)


class BetThree(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Three(bet_amount), mode=mode)


class BetYo(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Yo(bet_amount), mode=mode)


class BetBoxcars(_BaseSingleBet):
    def __init__(
        self,
        bet_amount: typing.SupportsFloat,
        mode=StrategyMode.ADD_IF_NOT_BET,
    ):
        super().__init__(Boxcars(bet_amount), mode=mode)


class BetFire(_BaseSingleBet):
    def __init__(self, bet_amount: float, mode=StrategyMode.ADD_IF_NOT_BET):
        super().__init__(Fire(bet_amount), mode=mode)


class BetAll(_BaseSingleBet):
    def __init__(self, bet_amount: float, mode=StrategyMode.ADD_IF_NOT_BET):
        super().__init__(All(bet_amount), mode=mode)


class BetTall(_BaseSingleBet):
    def __init__(self, bet_amount: float, mode=StrategyMode.ADD_IF_NOT_BET):
        super().__init__(Tall(bet_amount), mode=mode)


class BetSmall(_BaseSingleBet):
    def __init__(self, bet_amount: float, mode=StrategyMode.ADD_IF_NOT_BET):
        super().__init__(Small(bet_amount), mode=mode)



import os

import pytest

from crapssim import Table
from crapssim.strategy.odds import PassLineOddsMultiplier
from crapssim.strategy.single_bet import BetPassLine, BetPlace


def test_table_print_output(capsys):

    output_file = os.path.join(".", "tests", "integration", "verified_table_output.txt")

    table = Table(seed=8)
    table.add_player(bankroll=100, strategy=BetPassLine(5) + PassLineOddsMultiplier(2))
    table.add_player(bankroll=1000, strategy=BetPlace({5: 10, 6: 12, 8: 12, 4: 10}))
    table.run(max_rolls=20, verbose=True)

    # Capture and check output
    captured = capsys.readouterr()

    # # Do this once if the output changes
    # with open(output_file, "w") as f:
    #     f.write(captured.out)

    with open(output_file, "r") as f:
        output_content = f.read().strip()

    assert captured.out.rstrip() == output_content
